@@ -79,7 +79,9 @@ static	MySQLTypeMap	typeMap[] =
 {	FIELD_TYPE_TINY_BLOB,	KB::ITBinary,	"Tinyblob",	0			},
 {	FIELD_TYPE_BLOB,	KB::ITBinary,	"Blob",		0			},
 {	FIELD_TYPE_MEDIUM_BLOB,	KB::ITBinary,	"Mediumblob",	0			},
-{	FIELD_TYPE_LONG_BLOB,	KB::ITBinary,	"Longblob",	0			}
+{	FIELD_TYPE_LONG_BLOB,	KB::ITBinary,	"Longblob",	0			},
+
+{	-1,			KB::ITString,	"Text",		0			}
 }	;
 
 
@@ -1047,6 +1049,8 @@ bool	KBMySQL::doListFields
 		bool	     ai = (fields[idx].flags & AUTO_INCREMENT_FLAG) != 0 ;
 		bool	     uk = (fields[idx].flags & UNIQUE_KEY_FLAG    ) != 0 ;
 		bool	     ix = (fields[idx].flags & MULTIPLE_KEY_FLAG  ) != 0 ;
+		bool	     ts = (fields[idx].flags & TIMESTAMP_FLAG     ) != 0 ;
+		bool	     bf = (fields[idx].flags & BINARY_FLAG        ) != 0 ;
 
 #if	0
 		fprintf
@@ -1075,12 +1079,34 @@ bool	KBMySQL::doListFields
 			itype	= KB::ITUnknown	;
 		}
 
+		/* HACK: If the field is a timestamp then clear not-	*/
+		/*	 null. The column is marked not-null but we can	*/
+		/*	 insert null (and MySQL sets the timestamp).	*/
+		if (ts) nn = false ;
 
-		if (ai && (m->ident == FIELD_TYPE_LONG))
-			mtype	= "Primary Key" ;
+		switch (m->ident)
+		{
+			case FIELD_TYPE_BLOB	:
+				/* Blob without a binary flag is really	*/
+				/* a text column. Yuck.			*/
+				if (!bf)
+				{	mtype	= "Text"	;
+					itype	= KB::ITString	;
+				}
+				break	;
+		
+			case FIELD_TYPE_DECIMAL	:
+				fields[idx].length -= 2 ;
+				break	;
 
-		if (m->ident == FIELD_TYPE_DECIMAL)
-			fields[idx].length -= 2 ;
+			case FIELD_TYPE_LONG	:
+				/* Long (ie., integers) with the auto-	*/
+				/* increment flag is most treated as	*/
+				/* the "Primary Key" pseudo-type.	*/
+				if (ai)
+					mtype	= "Primary Key" ;
+				break	;
+		}
 
 		uint	flags	= 0 ;
 
