@@ -586,7 +586,7 @@ bool	KBPgSQL::doConnect
 	if (database.isEmpty())
 	{
 		database = "template1"	;
-		if (user.isEmpty()) user = "postgres" ;
+//		if (user.isEmpty()) user = "postgres" ;
 	}
 
 	if (!database.isEmpty())
