@@ -1,8 +1,9 @@
 /***************************************************************************
     file	         : kb_datetime.cpp
-    copyright            : (C) 1999,2000,2001,2002,2003,2004 by Mike Richardson
-			   (C) 2000,2001,2002,2003,2004 by theKompany.com
-			   (C) 2001,2002,2003,2004 by John Dean
+    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
+				 by Mike Richardson
+			   (C)	2001,2002,2003,2004,2005
+				by John Dean
     license              : This file is released under the terms of
                            the GNU General Public License, version 2. The
                            copyright holders retain the right to release
@@ -18,53 +19,17 @@
 #include	<qintdict.h>
 #include	<qlist.h>
 #include 	<qregexp.h>
+#include 	<qobject.h>
 
-#ifdef _WIN32
 
-int 	strcasecmp
-	(	const char	*s1,
-		const char	*s2
-	)
-{
-	while (*s1 && *s2)
-	{
-		char ch1 = *s1++ ;
-		char ch2 = *s2++ ;
-		if (isupper(ch1)) ch1 = tolower(ch1) ;
-		if (isupper(ch2)) ch2 = tolower(ch2) ;
-		if (ch1 != ch2) return ch1 - ch2 ;
-	}
-
-	return *s1 - *s2 ;
-}
-
-int	strncasecmp
-	(	const char	*s1,
-		const char	*s2,
-		size_t		len
-	)
-{
-	if(!len)
-		return(0);
-
-	while(--len && *s1 && *s2)
-	{
-		char ch1 = *s1++;
-		char ch2 = *s2++;
-
-		if(isupper(ch1))
-			ch1 = tolower(ch1);
-
-		if(isupper(ch2))
-			ch2 = tolower(ch2);
-
-		if(ch1 != ch2)
-			return ch1 - ch2;
-	}
-
-	return *s1 - *s2;
-}
+#define	__LOGGING	(0)
 
+#if	__LOGGING
+#define	LOGGING(x)	fprintf	(stderr, "KBDateTime::KBDateTime: ") ;	\
+			fprintf x ;					\
+			fprintf (stderr, "\n")			     ;
+#else
+#define	LOGGING(x)
 #endif
 
 #include	"kb_classes.h"
@@ -72,21 +37,31 @@ int	strncasecmp
 #include	"kb_value.h"
 
 
+#if		__KB_TKC
+class		KLocale	;
+#endif		// __KB_TKC
+
 #if		__KB_KDE
 
 #include	<klocale.h>
 #include	<kglobal.h>
 
-static	KLocale	*locale	;
-
 #endif		// __KB_KDE
 
 
+static	QStringList	weekDayNamesFull	;
+static	QStringList	weekDayNamesAbbrev	;
+static	QStringList	monthNamesFull		;
+static	QStringList	monthNamesAbbrev	;
+
+
 struct	LIBCOMMON_API	DTItem
-{	int	fchar	;
-	int	offset	;
-	int	rem	;
-	cchar	*fmt	;
+{
+	int	fchar	;	/* Format key character			*/
+	int	offset	;	/* Offset into date/time array		*/
+	int	rem	;	/* Display offset from data value	*/
+	cchar	*fmt	;	/* Format string			*/
+	int	len	;	/* Format output length			*/
 }	;
 
 #define	DT_YEAR		0
@@ -97,33 +72,35 @@ struct	LIBCOMMON_API	DTItem
 #define	DT_SEC		5
 #define	DT_YDAY		6
 #define	DT_WDAY		7
-#define	DT_SIZE		8
+#define	DT_AMPM		8
+#define	DT_SIZE		9
 
 #define	DT_FORMAT	99
 
 static	DTItem	dtItems[] =
 {
-	{	'C',	DT_YEAR,	-100,	"%02d"			},
-	{	'd',	DT_DAY,		0,	"%02d"			},
-	{	'e',	DT_DAY,		0,	"%d"			},
-	{	'H',	DT_HOUR,	0,	"%02d"			},
-	{	'I',	DT_HOUR,	12,	"%02d"			},
-	{	'j',	DT_YDAY,	0,	"%03d"			},
-	{	'k',	DT_HOUR,	0,	"%2d"			},
-	{	'l',	DT_HOUR,	12,	"%2d"			},
-	{	'm',	DT_MONTH,	0,	"%02d"			},
-	{	'M',	DT_MIN,		0,	"%02d"			},
-	{	'S',	DT_SEC,		0,	"%02d"			},
-	{	'u',	DT_WDAY,	0,	"%d"			},
-	{	'y',	DT_YEAR,	100,	"%02d"			},
-	{	'Y',	DT_YEAR,	0,	"%04d"			},
+{	'C',	DT_YEAR,	-100,	"%02d",			2	},
+{	'd',	DT_DAY,		0,	"%02d",			2	},
+{	'e',	DT_DAY,		0,	"%d",			0	},
+{	'H',	DT_HOUR,	0,	"%02d",			2	},
+{	'I',	DT_HOUR,	12,	"%02d",			2	},
+{	'j',	DT_YDAY,	0,	"%03d",			2	},
+{	'k',	DT_HOUR,	0,	"%2d",			2	},
+{	'l',	DT_HOUR,	12,	"%2d",			2	},
+{	'm',	DT_MONTH,	0,	"%02d",			2	},
+{	'M',	DT_MIN,		0,	"%02d",			2	},
+{	'S',	DT_SEC,		0,	"%02d",			2	},
+{	'u',	DT_WDAY,	0,	"%d",			0	},
+{	'y',	DT_YEAR,	100,	"%02d",			2	},
+{	'Y',	DT_YEAR,	0,	"%04d",			4	},
 
-	{	'c',	DT_FORMAT,	0,	"%x %X",		},
-	{	'D',	DT_FORMAT,	0,	"%m/%d/%y"		},
-	{	'r',	DT_FORMAT,	0,	"%I:%M:%S %p"		},
-	{	'R',	DT_FORMAT,	0,	"%H:%M"			},
-	{	'T',	DT_FORMAT,	0,	"%H:%M:%S"		},
-	{	'+',	DT_FORMAT,	0,	"%a %b %e %T %Z %Y"	}
+/* The following cases are recursive ....				*/
+{	'c',	DT_FORMAT,	0,	"%x %X",		-1	},
+{	'D',	DT_FORMAT,	0,	"%m/%d/%y",		-1	},
+{	'r',	DT_FORMAT,	0,	"%I:%M:%S %p",		-1	},
+{	'R',	DT_FORMAT,	0,	"%H:%M",		-1	},
+{	'T',	DT_FORMAT,	0,	"%H:%M:%S",		-1	},
+{	'+',	DT_FORMAT,	0,	"%a %b %e %T %Z %Y",	-1	}
 }	;
 
 static	QIntDict<DTItem>	dtMap	;
@@ -162,6 +139,7 @@ static	cchar	*names[] =
 	"nd",
 	"rd",
 	"th",
+	"ut",
 	0
 }	;
 #define	NUMDAYS	7
@@ -190,15 +168,14 @@ public	:
 
 
 /*  getName	: Locate name entry for specified string		*/
-/*  str		: cchar *	: String				*/
-/*  slen	: uint		: Number of relevant characters		*/
+/*  str		: cchar *	: String in question			*/
 /*  (returns)	: int		: Index in name list or negative	*/
 
 static	int	getName
-	(	cchar	*str,
-		uint	slen
+	(	cchar		*str
 	)
 {
+	int	slen	= qstrlen (str) ;
 	cchar	**nptr	;
 
 	for (nptr = &names[0] ; *nptr != 0 ; nptr += 1)
@@ -210,11 +187,11 @@ static	int	getName
 		/* length-dependant match. This distinguishes between,	*/
 		/* say, "thursday" and "th".				*/
 		if (nlen < 3)
-		{	if (strcasecmp (str, nam) == 0) break ;
+		{	if (qstricmp (str, nam) == 0) break ;
 			continue ;
 		}
 
-		if ((slen >= 3) && (strncasecmp (str, nam, slen) == 0))
+		if ((slen >= 3) && (qstrnicmp (str, nam, slen) == 0))
 			break	;
 	}
 
@@ -237,7 +214,143 @@ static	int	fixYear
 }
 
 
+/*  getLocale	: Get local information					*/
+/*  (returns)	: KLocale *	: Locale				*/
+
+static	KLocale	*getLocale ()
+{
+	for (uint idx = 0 ; idx < sizeof(dtItems)/sizeof(DTItem) ; idx += 1)
+		dtMap.insert (dtItems[idx].fchar, &dtItems[idx]) ;
+
 #if	__KB_KDE
+	static	KLocale	*locale	;
+
+	/* Set up on the first call. Try to figure the default date	*/
+	/* order. Format a date which can then be interpreted. If that	*/
+	/* does not help then the US is MDY and everywhere else is	*/
+	/* DMY.								*/
+	if (locale == 0)
+	{
+		locale	= KGlobal::locale () ;
+
+		QDate	dTest (2000, 10, 1)  ;
+		QString	dText = locale->formatDate (dTest, true) ;
+		int	dayno = dText.left(2).toInt() ;
+
+		fprintf
+		(	stderr,
+			"getLocale: (2000,10,1)->[%s]->[%d] co=[%s]\n",
+			dText.latin1(),
+			dayno,
+			locale->country().lower().latin1()
+		)	;
+
+		if	(dayno == 1 )
+		{
+			dateOrder = KBDateTime::DayMonthYear ;
+		}
+		else if (dayno == 10)
+		{
+			dateOrder = KBDateTime::MonthDayYear ;
+		}
+		else
+		{
+			dateOrder = locale->country().lower() == "us" ?
+						KBDateTime::MonthDayYear :
+						KBDateTime::DayMonthYear ;
+		}
+
+		fprintf
+		(	stderr,
+			"Date Format: [%s] [%s]\n",
+			(cchar *)dText,
+			dateOrder == KBDateTime::DayMonthYear ? "DMY" : "MDY"
+		)	;
+
+		for (uint day = 0 ; day <= 7 ; day += 1)
+		{	weekDayNamesAbbrev.append (getLocale()->weekDayName (day, true )) ;
+			weekDayNamesFull  .append (getLocale()->weekDayName (day, false)) ;
+		}
+		for (uint mon = 0 ; mon <= 12 ; mon += 1)
+		{	monthNamesAbbrev  .append (getLocale()->monthName   (mon, true )) ;
+			monthNamesFull    .append (getLocale()->monthName   (mon, false)) ;
+		}
+	}
+
+	return	locale	;
+#endif
+
+#if	__KB_TKC
+	static	bool	loaded	;
+
+	if (!loaded)
+	{
+		loaded	= true	;
+	
+		static	const char	*dayFull  [] =
+		{	0,
+			"Monday",
+			"Tuesday",
+			"Wednesday",
+			"Thursday",
+			"Friday",
+			"Saturday",
+			"Sunday"
+		}	;
+		static	const char	*dayAbbrev[] =
+		{	0,
+			"Mon",
+			"Tue",
+			"Wed",
+			"Thu",
+			"Fri",
+			"Sat",
+			"Sun"
+		}	;
+		static	const char	*monFull   [] =
+		{	0,
+			"January",
+			"February",
+			"March",
+			"April",
+			"May",
+			"June",
+			"July",
+			"August",
+			"September",
+			"October",
+			"November",
+			"December"
+		}	;
+		static	const char	*monAbbrev [] =
+		{	0,
+			"Jan",
+			"Feb",
+			"Mar",
+			"Apr",
+			"May",
+			"Jun",
+			"Jul",
+			"Aug",
+			"Sep",
+			"Oct",
+			"Nov",
+			"Dec",
+		}	;
+
+		for (uint day = 0 ; day <= 7 ; day += 1)
+		{	weekDayNamesAbbrev.append (dayAbbrev[day]) ;
+			weekDayNamesFull  .append (dayFull  [day]) ;
+		}
+		for (uint mon = 0 ; mon <= 12 ; mon += 1)
+		{	monthNamesAbbrev  .append (monAbbrev[mon]) ;
+			monthNamesFull    .append (monFull  [mon]) ;
+		}
+	}
+
+	return	0 ;
+#endif
+}
 
 /*  weekDayName	: Get name of day of week				*/
 /*  idx		: int		: Day number				*/
@@ -249,7 +362,12 @@ static	QString	weekDayName
 		bool		abbrev
 	)
 {
-	return	locale->weekDayName (idx, abbrev) ;
+	if ((idx >= 0) && (idx <= 7))
+		if (abbrev)
+			return	weekDayNamesAbbrev[idx] ;
+		else	return	weekDayNamesFull  [idx] ;
+
+	return	"???"	;
 }
 
 /*  monthName	: Get name of month					*/
@@ -262,9 +380,16 @@ static	QString	monthName
 		bool		abbrev
 	)
 {
-	return	locale->monthName (idx, abbrev) ;
+	if ((idx >= 0) && (idx <= 12))
+		if (abbrev)
+			return	monthNamesAbbrev[idx] ;
+		else	return	monthNamesFull  [idx] ;
+
+	return	"???"	;
 }
 
+#if	__KB_KDE
+
 /*  formatDate	: Get specified date in default format			*/
 /*  date	: const QDate &	: Date					*/
 /*  (returns)	: QString	: Formatted version			*/
@@ -273,7 +398,7 @@ static	QString	formatDate
 	(	const QDate	&date
 	)
 {
-	return	locale->formatDate (date) ;
+	return	getLocale()->formatDate (date) ;
 }
 
 /*  formatTime	: Get specified time in default format			*/
@@ -284,97 +409,13 @@ static	QString	formatTime
 	(	const QTime	&time
 	)
 {
-	return	locale->formatTime (time) ;
+	return	getLocale()->formatTime (time) ;
 }
 
 #endif	// __KB_KDE
 
 #if	__KB_TKC
 
-/*  weekDayName	: Get name of day of week				*/
-/*  idx		: int		: Day number				*/
-/*  abbrev	: bool		: Return abbreviated form		*/
-/*  (returns)	: QString	: Name					*/
-
-static	QString	weekDayName
-	(	int		idx,
-		bool		abbrev
-	)
-{
-	static	const char	*dayFull[] =
-	{	0,
-		"Monday",
-		"Tuesday",
-		"Wednesday",
-		"Thursday",
-		"Friday",
-		"Saturday",
-		"Sunday"
-	}	;
-	static	const char	*dayAbbrev[] =
-	{	0,
-		"Mon",
-		"Tue",
-		"Wed",
-		"Thu",
-		"Fri",
-		"Sat",
-		"Sun"
-	}	;
-
-	if (idx < 1) return "???" ;
-	if (idx > 7) return "???" ;
-
-	return	abbrev ? dayAbbrev[idx] : dayFull[idx] ;
-}
-
-/*  monthName	: Get name of month					*/
-/*  idx		: int		: Month number				*/
-/*  abbrev	: bool		: Return abbreviated form		*/
-/*  (returns)	: QString	: Name					*/
-
-static	QString	monthName
-	(	int		idx,
-		bool		abbrev
-	)
-{
-	static	const char	*monFull[] =
-	{	0,
-		"January",
-		"February",
-		"March",
-		"April",
-		"May",
-		"June",
-		"July",
-		"August",
-		"September",
-		"October",
-		"November",
-		"December"
-	}	;
-	static	const char	*monAbbrev[] =
-	{	0,
-		"Jan",
-		"Feb",
-		"Mar",
-		"Apr",
-		"May",
-		"Jun",
-		"Jul",
-		"Aug",
-		"Sep",
-		"Oct",
-		"Nov",
-		"Dec",
-	}	;
-
-	if (idx <  1) return "???" ;
-	if (idx > 12) return "???" ;
-
-	return	abbrev ? monAbbrev[idx] : monFull[idx] ;
-}
-
 /*  formatDate	: Get specified date in default format			*/
 /*  date	: const QDate &	: Date					*/
 /*  (returns)	: QString	: Formatted version			*/
@@ -409,37 +450,400 @@ static	QString	formatTime
 
 #endif	// __KB_TKC
 
-/*  KBDateTime								*/
-/*  KBDateTime	: Constructor for date-time object			*/
-/*  _dtstr	: const QString & : Date/Time string			*/
-/*  (returns)	: KBDateTime	  :					*/
+#define	__year	data[DT_YEAR ]
+#define	__mon	data[DT_MONTH]
+#define	__day	data[DT_DAY  ]
+#define	__hour	data[DT_HOUR ]
+#define	__mins	data[DT_MIN  ]
+#define	__sec	data[DT_SEC  ]
+#define	__ampm	data[DT_AMPM ]
 
-KBDateTime::KBDateTime
-	(	const QString	&_dtstr
+
+
+/*  KBDateTime								*/
+/*  doDeFormat	: Decode date/time string based on prior format		*/
+/*  toknum	: uint		   : Token from which to process	*/
+/*  tokens	: QList<DTToken> & : List of tokens			*/
+/*  fmt		: const QString &  : Format string			*/
+/*  data	: int[]		   : Array for results			*/
+/*  (returns)	: int		   : Next token or negative on error	*/
+
+int	KBDateTime::doDeFormat
+	(	int		toknum,
+		QList<DTToken>	&tokens,
+		const QString	&fmt,
+		int		data[]
+	)
+{
+	int	offset	= 0	;
+
+	for (uint d = 0 ; d < DT_SIZE ; d += 1) data[d] = -1 ;
+
+	while ((toknum < (int)tokens.count()) && (offset = fmt.find('%', offset)) >= 0)
+	{
+		LOGGING
+		((	stderr,
+			"doDeFormat: [%s]",
+			fmt.mid(offset, 2).latin1()
+		))	;
+
+		DTToken	*dt	= tokens.at(toknum) ;
+		char	ftch	= fmt[offset+1] ;
+		offset	+= 1	;
+
+		switch (ftch)
+		{
+			case 'a' :
+			case 'A' :
+				/* Actual day name. Not enough info so	*/
+				/* skip this token.			*/
+				toknum	+= 1	;
+				continue	;
+
+			case 'C' :
+				/* Century number is as much use as a	*/
+				/* chocolate tea-pot.			*/
+				toknum	+= 1	;
+				continue	;
+
+			case 'b' :
+			case 'h' :
+			case 'B' :
+				/* Month name, store as the month	*/
+				/* number and advance the tokens	*/
+				data[DT_MONTH] = dt->nidx - NUMDAYS + 1 ;
+				toknum	+= 1	;
+				continue	;
+
+			case 'n' :
+			case 't' :
+			case '%' :
+				/* Ignore these, they are just escapes.	*/
+				continue	;
+
+			default	:
+				break	;
+		}
+
+		/* Not a special case, so look for the format character	*/
+		/* in the type map. If not found then silently ignore	*/
+		/* it.							*/
+		DTItem	*di	= dtMap.find (ftch) ;
+
+		LOGGING
+		((	stderr,
+			"doDeFormat: [%c]->[%p] from [%s]",
+			ftch,
+			(void *)di,
+			dt->token.latin1()
+		))	;
+
+		if (di == 0)
+			continue ;
+
+		/* Specical case here is DT_FORMAT, in which case we	*/
+		/* recursively process the format string.		*/
+		if (di->offset == DT_FORMAT)
+		{
+			toknum	= doDeFormat (toknum, tokens, di->fmt, data) ;
+			if (toknum < 0) return toknum ;
+			continue  ;
+		}
+
+		/* Get the numerical value. Then do some range checks	*/
+		/* to make sure we have sensible values.		*/
+		int	v = dt->token.toInt() ;
+		switch (di->offset)
+		{
+			case DT_YEAR	:
+				if (v >= 0)
+					if	(v <=  50) v += 2000 ;
+					else if	(v <  100) v += 1900 ;
+				break	;
+
+			case DT_HOUR	:
+				v += di->rem ;
+				break	;
+
+			default	:
+				break	;
+		}
+
+		data[di->offset] = v ;
+		toknum	+= 1 ;
+	}
+
+	/* Note that the actual decoded values will be checked in the	*/
+	/* ::decodeOK() method, so if we have decoded (say) a day value	*/
+	/* 32, this will be picked up later.				*/
+	return	toknum	;
+}
+
+/*  KBDateTime								*/
+/*  doDecode	: Decode date/time without using a format		*/
+/*  tokens	: QList<DTToken> & : List of tokens			*/
+/*  data	: int[]		   : Array for results			*/
+/*  (returns)	: bool		   : Success				*/
+
+bool	KBDateTime::doDecode
+	(	QList<DTToken>	&tokens,
+		int		data[]
 	)
 {
 #define	SF(f,v)	\
-	{	if (f != -1) return ;	\
-		f = v ;			\
+	{	if (f != -1) return false ;	\
+		f = v ;				\
 	}
 
-	int		year	= -1	;
-	int		mon	= -1	;
-	int		day	= -1	;
-	int		hour	= -1	;
-	int		min	= -1	;
-	int		sec	= -1	;
-	int		ampm	= -1	;
-	int		lch	= ' '	;
-	int		left	;
+	uint	tokp	= 0 ;
+	int	left	;
+
+	for (uint d = 0 ; d < DT_SIZE ; d += 1) data[d] = -1 ;
+
+	while ((left = tokens.count() - tokp) > 0)
+	{
+		DTToken	*tok0	= tokens.at(tokp) ;
+		DTToken	*tok1	= left > 1 ? tokens.at(tokp + 1) : 0 ;
+		DTToken	*tok2	= left > 2 ? tokens.at(tokp + 2) : 0 ;
+		DTToken	*tok3	= left > 3 ? tokens.at(tokp + 3) : 0 ;
+		uint	drop	= 0 ;
+
+		LOGGING	((stderr, " .... token [%s] with %d remaining", (cchar *)tok0->token, left)) ;
+
+		/* First set of cases are where there are three tokens	*/
+		/* available. Look for dates, and times where the	*/
+		/* number of seconds is set.				*/
+		if (tok2 != 0)
+		{
+			/* NN:NN:NN					*/
+			/* Decode this as a time with the value order	*/
+			/* of (hour, minute, second).			*/
+			if	(tok0->digit &&  tok1->digit && tok2->digit
+					&& (tok1->sep == ':') && (tok2->sep == ':'))
+			{
+				SF(__hour, tok0->nidx) ;
+				SF(__mins, tok1->nidx) ;
+				SF(__sec,  tok2->nidx) ;
+
+				if ((tok3 != 0) && (tok3->sep == '.'))
+					drop	= 4  ;
+				else	drop	= 3  ;
+				goto	advance	;
+			}
+			/* NN-CCC-NN					*/
+			/* This is a date in order (day,month,year)	*/
+			/* where the month has been named.		*/
+			else if (tok0->digit && !tok1->digit && tok2->digit
+					&& (tok1->sep == '-') && (tok2->sep == '-'))
+			{
+				
+				SF(__day,  tok0->nidx   ) ;
+				SF(__mon,  tok1->nidx - NUMDAYS + 1) ;
+				SF(__year, fixYear(tok2)) ;
+				drop	= 3	;
+				goto	advance	;
+			}
+			/* NN-NN-NN					*/
+			/* NN/NN/NN					*/
+			/* NN.NN.NN					*/
+			/* Decode thiese as a date. If the first token	*/
+			/* is more then two characters long then treat	*/
+			/* it as (year,month, day). Otherwise, treat as	*/
+			/* (day,month,year) or (month,day,year)		*/
+			/* depending on the default date ordering.	*/
+			else if (tok0->digit &&  tok1->digit && tok2->digit
+					&& (tok1->sep == tok2->sep)
+					&& ( (tok1->sep == '-') ||
+					     (tok1->sep == '/') ||
+					     (tok1->sep == '.') ) )
+			{
+				if	(tok0->token.length() > 2)
+				{
+					SF(__year, fixYear(tok0)) ;
+					SF(__mon,  tok1->nidx   ) ;
+					SF(__day,  tok2->nidx   ) ;
+				}
+				else if	(dateOrder == KBDateTime::DayMonthYear)
+				{
+					SF(__year, fixYear(tok2)) ;
+					SF(__mon,  tok1->nidx   ) ;
+					SF(__day,  tok0->nidx   ) ;
+				}
+				else
+				{
+					SF(__year, fixYear(tok2)) ;
+					SF(__mon,  tok0->nidx   ) ;
+					SF(__day,  tok1->nidx   ) ;
+				}
+
+				drop	= 3	;
+				goto	advance	;
+			}
+		}
+
+		/* Not three tokens or nothing matched, next check for	*/
+		/* a two-token case, which can only be a time with the	*/
+		/* seconds unspecified (and interpreted as zero).	*/
+		if (tok1 != 0)
+			if	(tok0->digit && tok1->digit && (tok1->sep == ':'))
+			{
+				SF(__hour, tok0->nidx) ;
+				SF(__mins, tok1->nidx) ;
+				SF(__sec,  0) ;
+				drop	= 2	;
+				goto	advance ;
+			}
+
+		/* Allow three and four digit tokens which have a	*/
+		/* leading + or -; these are GMT offsets. For the	*/
+		/* moment, just skip them.				*/
+		if ((tok0->token.length() >= 3) && (tok0->token.length() <= 4))
+			if ((tok0->sep == '+') || (tok0->sep == '-'))
+			{
+				drop	= 1	;
+				goto	advance	;
+			}
+
+		/* Only one token or again, no matches. Days are of	*/
+		/* no use, but check for months or for AM/PM.		*/
+		if (!tok0->digit)
+		{
+			if	(tok0->nidx <  NUMDAYS)
+				;
+			else if	(tok0->nidx <  NUMDAYS+NUMMONS)
+				SF(__mon,  tok0->nidx - NUMDAYS + 1)
+			else if (tok0->nidx == NUMISAM)
+				SF(__ampm, NUMISAM)
+			else if (tok0->nidx == NUMISPM)
+				SF(__ampm, NUMISPM)
+
+			drop	= 1	;
+			goto	advance	;
+		}
+
+		/* Two digit numbers are interpreted as day numbers, so	*/
+		/* we can handle things like '11 July'.			*/
+		if (tok0->token.length() <= 2)
+		{
+			if ((tok0->sep != '+') && (tok0->sep != '-'))
+				SF(__day,  tok0->nidx) ;
+			drop	= 1	;
+			goto	advance	;
+		}
+
+		/* Four digit tokens can only be years ...		*/
+		if ((tok0->token.length() <= 4) && tok0->digit)
+		{
+			SF(__year, tok0->nidx) ;
+			drop	= 1	;
+			goto	advance	;
+		}
+
+		/* ... and anything else ends the scan.			*/
+		LOGGING	((stderr, "fail on [%s]", (cchar *)tok0->token)) ;
+		return	false	;
+
+		advance	:
+			tokp	+= drop ;
+	}
+
+	/* Note that the actual decoded values will be checked in the	*/
+	/* ::decodeOK() method, so if we have decoded (say) a day value	*/
+	/* 32, this will be picked up later.				*/
+	return	true	;
+}
+
+/*  KBDateTime								*/
+/*  decodeOK	: Check that the decoded values are OK			*/
+/*  data	: int[]		   : Array of decoded results		*/
+/*  (returns)	: bool		   : Success				*/
+
+bool	KBDateTime::decodeOK
+	(	int		data[]
+	)
+{
+	/* This method does the actual checking. The decoder methods	*/
+	/* are responsible for breaking up the date text; this method	*/
+	/* checks that the results are plausible.			*/
+	m_hasDate = false	;
+	m_hasTime = false	;
+
+	LOGGING
+	((	stderr,
+		" .... decoded: hour=%d ampm=%d",
+		__hour,
+		__ampm
+	))	;
+
+	if (__hour != -1)
+	{
+		if ((__ampm != -1) && (__hour >= 12))
+			return	false	;
+
+		if (__ampm == NUMISPM)
+			__hour += 12	;
+	}
+
+	LOGGING
+	((	stderr,
+		" .... decoded: %d-%d-%d %d:%d:%d",
+		__year, __mon,  __day,
+		__hour, __mins, __sec
+	))	;
+
+	if ((__year != -1) && (__mon != -1) && (__day != -1))
+	{
+		if ((__year < 1752) || (__year > 8000)) return false ;
+		if ((__mon  <    1) || (__mon  >   12)) return false ;
+		if ( __day  <    1		      ) return false ;
+
+		mDays[1] = __year % 4   != 0 ? 28 :
+			   __year % 400 == 0 ? 29 :
+			   __year % 100 == 0 ? 28 : 29 ;
+
+		if ( __day  > mDays[__mon-1])
+			return	false	;
+
+		m_dt.setDate (QDate(__year, __mon, __day)) ;
+		m_hasDate = true ;
+	}
+	else	m_dt.setDate (QDate()) ;
+
+	if ((__hour != -1) && (__mins != -1) && (__sec != -1))
+	{
+		if (__hour >= 24) return false ;
+		if (__mins >= 60) return false ;
+		if (__sec  >= 60) return false ;
+
+		m_dt.setTime (QTime(__hour, __mins, __sec)) ;
+		m_hasTime = true ;
+	}
+	else	m_dt.setTime (QTime()) ;
+
+	return	true		;
+}
+
+/*  KBDateTime								*/
+/*  KBDateTime	: Constructor for date-time object			*/
+/*  _dtstr	: const QString & : Date/Time string			*/
+/*  fmt		: const QString & : Format				*/
+/*  (returns)	: KBDateTime	  :					*/
+
+KBDateTime::KBDateTime
+	(	const QString	&_dtstr,
+		const QString	&fmt
+	)
+{
+
+	int		lch	= ' ' ;
 	QList<DTToken>	tokens	;
 	DTToken		*tokp	;
 
-//	fprintf
-//	(	stderr,
-//		"KBDateTime::KBDateTime: [%s]\n",
-//		(cchar *)_dtstr
-//	)	;
+	/* Ensure that all date information is set up before we use any	*/
+	/* The "getLocale()" routine does this as a side effect.	*/
+	getLocale() ;
+
+	LOGGING	((stderr, "[%s]", (cchar *)_dtstr)) ;
 
 	/* Duplicate the date/time string and store it as the raw data.	*/
 	/* This copy is also used in the parser below, but is left	*/
@@ -475,260 +879,105 @@ KBDateTime::KBDateTime
 
 		tokp->token	= QString(tok).left(dtstr - tok) ;
 
+		LOGGING	((stderr, "token [%s]", (cchar *)tokp->token))
+
 		if (!tokp->digit)
 		{
-			if ((tokp->nidx = getName(tok, dtstr - tok)) < 0)
+			if ((tokp->nidx = getName((cchar *)tokp->token)) < 0)
+			{
+				LOGGING	((stderr, "failed on token [%s]", (cchar *)tokp->token)) ;
 				return	;
+			}
 		}
 		else	tokp->nidx = atol (tok) ;
 	}
 
-//	fprintf
-//	(	stderr,
-//		"KBDateTime::KBDateTime: [%s] -> %d tokens\n",
-//		(cchar *)_dtstr,
-//		tokens.count()
-//	)	;
-//	LITER
-//	(	DTToken,
-//		tokens,
-//		tokp,
-//		fprintf (stderr, " .... %c %s %d\n", tokp->sep, (cchar *)tokp->token, tokp->nidx) ;
-//	)
+#if	__LOGGING
+	LOGGING	((stderr, "[%s] -> %d tokens", (cchar *)_dtstr, tokens.count())) ;
+	LITER
+	(	DTToken,
+		tokens,
+		tokp,
+		LOGGING	((stderr, ".... %c %s %d", tokp->sep, (cchar *)tokp->token, tokp->nidx)) ;
+	)
+#endif
 
-	/* Special case: this is really a hack for XBase files, where	*/
-	/* dates are stored as YYYYMMDD, and there is no time component	*/
+	/* If there is a single token then it may be something like	*/
+	/* "20050111...." comprising year-month-day and perhaps		*/
+	/* hour-minute-second. This must not have any non-digits.	*/
 	if (tokens.count() == 1)
 	{
-		static	QRegExp	xbdate	("^[0-9][0-9][0-9][0-9][0-9][0-9]$") ;
-
 		const QString	&dstr	= tokens.at(0)->token ;
+		int		data[DT_SIZE]	;
+		for (uint d = 0 ; d < DT_SIZE ; d += 1) data[d] = -1 ;
 
-		if (xbdate.match (dstr))
+		/* Assume all digits, but no year-month-day nor		*/
+		/* hour-minute-second parts. Then scan the token to	*/
+		/* see what we actually have.				*/
+		bool		alld	= true	;
+		bool		ymd	= false	;
+		bool		hms	= false	;
+
+		for (uint idx = 0 ; idx < dstr.length() ; idx += 1)
 		{
-			year	= dstr.mid(0, 4).toInt() ;
-			mon	= dstr.mid(4, 2).toInt() ;
-			day	= dstr.mid(6, 2).toInt() ;
+			if (!dstr.at(idx).isDigit())
+			{	alld	= false	;
+				break	;
+			}
+			if (idx ==  7) ymd = true ;
+			if (idx == 13) hms = true ;
+		}
 
-			m_dt.setDate (QDate(year, mon, day)) ;
-			m_dt.setTime (QTime()) ;
-
-			m_hasDate = true  ;
-			m_hasTime = false ;
-			m_valid	  = true  ;
+		/* Unless all characters are digits and we have a valid	*/
+		/* year-month-day, then treat it as unknown.		*/
+		if (!alld || !ymd)
+		{
+			LOGGING	((stderr, " .... single unknown token")) ;
 			return	;
 		}
 
+		/* Decode the year-month-day, which we know we have,	*/
+		/* and the hour-minute-second if we have that.		*/
+		__year	= dstr.mid( 0, 4).toInt() ;
+		__mon	= dstr.mid( 4, 2).toInt() ;
+		__day	= dstr.mid( 6, 2).toInt() ;
+
+		LOGGING	((stderr, " .... YMD %04d-%02d-%02d", __year, __mon, __day)) ;
+
+		m_dt.setDate (QDate(__year, __mon, __day)) ;
+		m_hasDate = true  ;
+
+		if (hms)
+		{	__hour	= dstr.mid( 8, 2).toInt() ;
+			__mins	= dstr.mid(10, 2).toInt() ;
+			__sec	= dstr.mid(12, 2).toInt() ;
+
+			LOGGING	((stderr, " .... HMS %02d-%02d-%02d", __hour, __mins, __sec)) ;
+
+			m_dt.setTime (QTime(__hour, __mins, __sec)) ;
+			m_hasTime = true ;
+		}
+
+		m_valid	  = true  ;
 		return	;
 	}
 
-	while ((left = tokens.count()) > 0)
+	bool	ok	= false ;
+	int	data[DT_SIZE]	;
+
+	if (!fmt.isEmpty())
 	{
-		DTToken	*tok0	= tokens.at(0) ;
-		DTToken	*tok1	= left > 1 ? tokens.at(1) : 0 ;
-		DTToken	*tok2	= left > 2 ? tokens.at(2) : 0 ;
-		DTToken	*tok3	= left > 3 ? tokens.at(3) : 0 ;
-		uint	drop	= 0 ;
-
-//		fprintf
-//		(	stderr,
-//			" .... token [%s] with %d remaining\n",
-//			(cchar *)tok0->token,
-//			left
-//		)	;
-
-		/* First set of cases are where there are three tokens	*/
-		/* available. Look for dates, and times where the	*/
-		/* number of seconds is set.				*/
-		if (tok2 != 0)
-		{
-			/* NN:NN:NN					*/
-			/* Decode this as a time with the value order	*/
-			/* of (hour, minute, second).			*/
-			if	(tok0->digit &&  tok1->digit && tok2->digit
-					&& (tok1->sep == ':') && (tok2->sep == ':'))
-			{
-				SF(hour, tok0->nidx) ;
-				SF(min,  tok1->nidx) ;
-				SF(sec,  tok2->nidx) ;
-
-				if ((tok3 != 0) && (tok3->sep == '.'))
-					drop	= 4  ;
-				else	drop	= 3  ;
-				goto	advance	;
-			}
-			/* NN-CCC-NN					*/
-			/* This is a date in order (day,month,year)	*/
-			/* where the month has been named.		*/
-			else if (tok0->digit && !tok1->digit && tok2->digit
-					&& (tok1->sep == '-') && (tok2->sep == '-'))
-			{
-				
-				SF(day,  tok0->nidx   ) ;
-				SF(mon,  tok1->nidx - NUMDAYS + 1) ;
-				SF(year, fixYear(tok2)) ;
-				drop	= 3	;
-				goto	advance	;
-			}
-			/* NN-NN-NN					*/
-			/* NN/NN/NN					*/
-			/* NN.NN.NN					*/
-			/* Decode thiese as a date. If the first token	*/
-			/* is more then two characters long then treat	*/
-			/* it as (year,month, day). Otherwise, treat as	*/
-			/* (day,month,year) or (month,day,year)		*/
-			/* depending on the default date ordering.	*/
-			else if (tok0->digit &&  tok1->digit && tok2->digit
-					&& (tok1->sep == tok2->sep)
-					&& ( (tok1->sep == '-') ||
-					     (tok1->sep == '/') ||
-					     (tok1->sep == '.') ) )
-			{
-				if	(tok0->token.length() > 2)
-				{
-					SF(year, fixYear(tok0)) ;
-					SF(mon,  tok1->nidx   ) ;
-					SF(day,  tok2->nidx   ) ;
-				}
-				else if	(dateOrder == KBDateTime::DayMonthYear)
-				{
-					SF(year, fixYear(tok2)) ;
-					SF(mon,  tok1->nidx   ) ;
-					SF(day,  tok0->nidx   ) ;
-				}
-				else
-				{
-					SF(year, fixYear(tok2)) ;
-					SF(mon,  tok0->nidx   ) ;
-					SF(day,  tok1->nidx   ) ;
-				}
-
-				drop	= 3	;
-				goto	advance	;
-			}
-		}
-
-		/* Not three tokens or nothing matched, next check for	*/
-		/* a two-token case, which can only be a time with the	*/
-		/* seconds unspecified (and interpreted as zero).	*/
-		if (tok1 != 0)
-			if	(tok0->digit && tok1->digit && (tok1->sep == ':'))
-			{
-				SF(hour, tok0->nidx) ;
-				SF(min,  tok1->nidx) ;
-				SF(sec,  0) ;
-				drop	= 2	;
-				goto	advance ;
-			}
-
-		/* Allow three and four digit tokens which have a	*/
-		/* leading + or -; these are GMT offsets. For the	*/
-		/* moment, just skip them.				*/
-		if ((tok0->token.length() >= 3) && (tok0->token.length() <= 4))
-			if ((tok0->sep == '+') || (tok0->sep == '-'))
-			{
-				drop	= 1	;
-				goto	advance	;
-			}
-
-		/* Only one token or again, no matches. Days are of	*/
-		/* no use, but check for months or for AM/PM.		*/
-		if (!tok0->digit)
-		{
-			if	(tok0->nidx <  NUMDAYS)
-				;
-			else if	(tok0->nidx <  NUMDAYS+NUMMONS)
-				SF(mon,  tok0->nidx - NUMDAYS + 1)
-			else if (tok0->nidx == NUMISAM)
-				SF(ampm, NUMISAM)
-			else if (tok0->nidx == NUMISPM)
-				SF(ampm, NUMISPM)
-
-			drop	= 1	;
-			goto	advance	;
-		}
-
-		/* Two digit numbers are interpreted as day numbers, so	*/
-		/* we can handle things like '11 July'.			*/
-		if (tok0->token.length() <= 2)
-		{
-			if ((tok0->sep != '+') && (tok0->sep != '-'))
-				SF(day,  tok0->nidx) ;
-			drop	= 1	;
-			goto	advance	;
-		}
-
-		/* Four digit tokens can only be years ...		*/
-		if ((tok0->token.length() <= 4) && tok0->digit)
-		{
-			SF(year, tok0->nidx) ;
-			drop	= 1	;
-			goto	advance	;
-		}
-
-		/* ... and anything else ends the scan.			*/
-//		fprintf
-//		(	stderr,
-//			"KBDateTime::KBDateTime: fail on [%s]\n",
-//			(cchar *)tok0->token
-//		)	;
-		return	;
-
-		advance	:
-			while (drop > 0)
-			{	tokens.remove ((uint)0) ;
-				drop	-= 1 ;
-			}
+		if (doDeFormat (0, tokens, fmt, data) >= 0)
+			ok = decodeOK (data) ;
 	}
 
-//	fprintf
-//	(	stderr,
-//		" .... decoded: hour=%d ampm=%d\n",
-//		hour,
-//		ampm
-//	)	;
-
-	if (hour != -1)
-	{	if (ampm != 	  -1) if (hour >= 12) return ;
-		if (ampm ==  NUMISPM) hour += 12 ;
-	}
-
-
-//	fprintf
-//	(	stderr,
-//		" .... decoded: %d-%d-%d %d:%d:%d\n",
-//		year, mon, day,
-//		hour, min, sec
-//	)	;
-
-	if ((year != -1) && (mon != -1) && (day != -1))
+	if (!ok)
 	{
-		if ((year < 1752) || (year > 8000)) return ;
-		if ((mon  <    1) || (mon  >   12)) return ;
-		if ( day  <    1		  ) return ;
-
-		mDays[1] = year % 4   != 0 ? 28 :
-			   year % 400 == 0 ? 29 :
-			   year % 100 == 0 ? 28 : 29 ;
-
-
-		if ( day  > mDays[mon-1]) return ;
-
-		m_dt.setDate (QDate(year, mon, day)) ;
-		m_hasDate = true ;
+		if (doDecode   (tokens, data))
+			ok = decodeOK (data) ;
 	}
-	else	m_dt.setDate (QDate()) ;
 
-	if ((hour != -1) && (min != -1) && (sec != -1))
-	{
-		if (hour >= 24) return ;
-		if (min  >= 60) return ;
-		if (sec  >= 60) return ;
-
-		m_dt.setTime (QTime(hour, min, sec)) ;
-		m_hasTime = true ;
-	}
-	else	m_dt.setTime (QTime()) ;	
+	if (!ok) return	;
 
 	m_valid	= m_hasDate || m_hasTime ;
 }
@@ -753,42 +1002,10 @@ QString	KBDateTime::format
 	)
 	const
 {
-	if (dtMap.count() == 0)
-	{
-		for (uint idx = 0 ; idx < sizeof(dtItems)/sizeof(DTItem) ; idx += 1)
-			dtMap.insert (dtItems[idx].fchar, &dtItems[idx]) ;
+	/* Ensure that all date information is set up before we use	*/
+	/* any The "getLocale()" routine does this as a side effect.	*/
+	getLocale() ;
 
-#if	__KB_KDE
-		locale	= KGlobal::locale () ;
-
-		/* Try to figure the default date order. Format a date	*/
-		/* which can then be interpreted. If that does not	*/
-		/* help then the US is MDY and everywhere else is DMY.	*/
-		QDate	dTest (2000, 10, 1)  ;
-		QString	dText = locale->formatDate (dTest, true) ;
-		int	dayno = dText.left(2).toInt() ;
-
-		if	(dayno == 1 )
-		{
-			dateOrder = DayMonthYear ;
-		}
-		else if (dayno == 10)
-		{
-			dateOrder = MonthDayYear ;
-		}
-		else
-		{
-			dateOrder = locale->country().lower() == "us" ? MonthDayYear : DayMonthYear ;
-		}
-
-		fprintf
-		(	stderr,
-			"Date Format: [%s] [%s]\n",
-			(cchar *)dText,
-			dateOrder == DayMonthYear ? "DMY" : "MDY"
-		)	;
-#endif
-	}
 
 	/* If the decoded date/time is marked as invalid then make do	*/
 	/* with the original raw text.					*/
@@ -856,9 +1073,9 @@ QString	KBDateTime::format
 				tmp    	= monthName   (idx, fc != 'B') ;
 				break	;
 
-			case 'D' :
-				tmp	= format ("%m/%d/%y") ;
-				break	;
+//			case 'D' :
+//				tmp	= format ("%m/%d/%y") ;
+//				break	;
 
 			case 'n' :
 				tmp	= "\n"	;
@@ -949,9 +1166,11 @@ QString	KBDateTime::defFormat
 }
 
 
+
 void	KBDateTime::setDateOrder
 	(	DateOrder	_dateOrder
 	)
 {
 	dateOrder = _dateOrder	;
 }
+
