@@ -1133,11 +1133,15 @@ QString	KBServer::doMapExpression
 
 		while (ch.isLetterOrNumber() || (spec.find(ch) >= 0) || (spaceOK && (ch == ' ')))
 		{
-			/* If the character is a space and we have not	*/
-			/* hit a non-digit character then we have a	*/
-			/* number, so drp out of the loop.		*/
-			if ((ch == ' ') && !nonDigit)
-				break	;
+			/* If the character is a space then drop out of	*/
+			/* the loop if either (a) we have not hit a	*/
+			/* non-digit (so we have a number) or (b) the	*/
+			/* token is a keyword.				*/
+			if (ch == ' ')
+			{
+				if (!nonDigit) break ;
+				if (keywords->find(token.latin1()) != 0) break ;
+			}
 
 			/* If the character is not a digit and not a	*/
 			/* space then set the "nonDigit" flag. We use	*/
