@@ -202,10 +202,9 @@ static	KBFormatInfo	*getFormatInfo
 /*  text	: const QString & : Text to deformat			*/
 /*  type	: KBType *	  : Database type			*/
 /*  fmt		: const QString & : Format string			*/
-/*  (returns)	: const QString & : Deformatted text			*/
+/*  (returns)	: QString	  : Deformatted text			*/
 
-const QString
-	&KBValue::deFormat
+QString	KBValue::deFormat
 	(	const QString	&text,
 		KBType		*type,
 		const QString	&fmt
@@ -239,7 +238,7 @@ const QString
 		case KB::ITDateTime	:
 			/* Assume that any date/time formatting leaves	*/
 			/* the text in a recognisable format.		*/
-			return	text	;
+			return	KBDateTime(text, fmt).defFormat(type->getIType()) ;
 
 		case KB::ITBinary	:
 		case KB::ITDriver	:
@@ -1281,11 +1280,29 @@ QString	KBValue::getText
 	int	cs	;
 
 	if ((cs = fmt.find (':')) >= 0)
-	{	fType	= fmt.left (cs) ;
+	{
+		fType	= fmt.left (cs) ;
 		fForm	= fmt.mid  (cs + 1) ;
+
+		if (fType[0] == '!')
+		{
+			KBType	*t	;
+			if	(fType == "!String"  ) t = &_kbString	;
+			else if (fType == "!Number"  ) t = &_kbFloat	;
+			else if (fType == "!Currency") t = &_kbFloat	;
+			else if (fType == "!Fixed"   ) t = &_kbFixed	;
+			else if (fType == "!Float"   ) t = &_kbFloat	;
+			else if (fType == "!Date"    ) t = &_kbDate	;
+			else if (fType == "!Time"    ) t = &_kbTime	;
+			else if (fType == "!DateTime") t = &_kbDateTime	;
+			else			       t = &_kbString	;
+			KBValue	v2 (*this, t) ;
+			return	v2.getText (fmt.mid(1), ok) ;
+		}
 	}
 	else if (!fmt.isEmpty())
-	{	if (ok != 0) *ok = false ;
+	{
+		if (ok != 0) *ok = false ;
 		return	"format?" + fmt  ;
 	}
 
