@@ -60,6 +60,7 @@ KBFormBlock::KBFormBlock
 	m_curItem  = 0	     ;
 	m_inQuery  = false   ;
 	m_dChanged = false   ;
+	m_prepared = false   ;
 
 	if ((dx.getIntValue() == 0) && (dy.getIntValue() == 0))
 			dy.setValue (25) ;
@@ -92,6 +93,7 @@ KBFormBlock::KBFormBlock
 	m_curItem  = 0	     ;
 	m_inQuery  = false   ;
 	m_dChanged = false   ;
+	m_prepared = false   ;
 }
 
 /*  KBFormBlock								*/
@@ -115,6 +117,7 @@ KBFormBlock::KBFormBlock
 	m_curItem  = 0	     ;
 	m_inQuery  = false   ;
 	m_dChanged = false   ;
+	m_prepared = false   ;
 }
 
 /*  KBFormBlock								*/
@@ -134,7 +137,8 @@ void	KBFormBlock::showAs
 	(	KB::ShowAs	mode
 	)
 {
-	KBBlock::showAs (mode) ;
+	m_prepared = false   	;
+	KBBlock::showAs (mode)	;
 
 	if (mode == KB::ShowAsData) 
 	{
@@ -148,6 +152,12 @@ void	KBFormBlock::showAs
 		blkDisp->setTitle (title.getValue()) ;
 }
 
+void	KBFormBlock::prepare ()
+{
+	m_prepared = true	;
+	KBBlock::prepare ()	;
+}
+
 /*  KBFormBlock								*/
 /*  newTabOrder	: Invoke tab order dialog				*/
 /*  (returns)	: void		:					*/
