@@ -758,9 +758,12 @@ void	KBFramer::showAs
 
 /*  KBFramer								*/
 /*  writeData	 : Display framer contents				*/
+/*  prior	 : bool		: Write prior value			*/
 /*  (returns)	 : bool		: Success				*/
 
-bool	KBFramer::writeData ()
+bool	KBFramer::writeData
+	(	bool		prior
+	)
 {
 	QRect	 rect    (QPoint(0, 0), geometry().size()) ;
 	KBWriter *writer = getReport()->getWriter() ;
@@ -771,7 +774,7 @@ bool	KBFramer::writeData ()
 	CITER
 	(	Object,
 		o,
-		if (!o->write (writer, QPoint(0,0), true, extra))
+		if (!o->write (writer, QPoint(0,0), true, extra, prior))
 			return false ;
 	)
 
@@ -793,13 +796,15 @@ int	KBFramer::height ()
 /*  offset	: QPoint	: Offset in writer			*/
 /*  first	: bool		: First value flag			*/
 /*  extra	: int &		: Return extra space			*/
+/*  prior	: bool		: Write prior value			*/
 /*  (returns)	: bool		: Success				*/
 
 bool	KBFramer::write
 	(	KBWriter	*writer,
 		QPoint		offset,
 		bool		first,
-		int		&extra
+		int		&extra,
+		bool		prior
 	)
 {
 	QString	bgcolor	;
@@ -812,7 +817,7 @@ bool	KBFramer::write
 #endif
 
 	QPoint	s = writer->setOffset (false, QPoint (geometry().topLeft())) ;
-	KBNode::write (writer, offset, first, extra) ;
+	KBNode::write (writer, offset, first, extra, prior) ;
 	writer->setOffset (true, s)  ;
 	return	true	;
 }
