@@ -1434,6 +1434,8 @@ void	KBObject::insertComponent
 	clink->showAs       (KB::ShowAsDesign) ;
 }
 
+#endif
+
 /*  KBObject								*/
 /*  findAllConfigs							*/
 /*		: Locate all configuration objects in document		*/
@@ -1469,8 +1471,6 @@ void	KBObject::findAllConfigs
 	)
 }
 
-#endif
-
 
 /*  KBObject								*/
 /*  pasteObjects: Paste in cut or copied objects			*/
@@ -1754,13 +1754,15 @@ KBNavigator *KBObject::getNavigator ()
 /*  offset	: QPoint	: Geometry offset			*/
 /*  first	: bool		: First on page				*/
 /*  extra	: init &	: Return extra space			*/
+/*  prior	: bool		: Write prior value			*/
 /*  (returns)	: void		:					*/
 
 bool	KBObject::write
 	(	KBWriter	*writer,
 		QPoint		offset,
 		bool		,
-		int		&extra
+		int		&extra,
+		bool		
 	)
 {
 	/* The work is all done at the control level.			*/
