@@ -348,10 +348,12 @@ void	KBSelectExpr::print ()
 /*  KBSelectExpr							*/
 /*  exprText	: Get expression text					*/
 /*  dbLink	: KBDBlink *	: Database link				*/ 
+/*  pretty	: bool		: Used in pretty text			*/
 /*  (returns)	: QString	: Text					*/
 
 QString	KBSelectExpr::exprText
-	(	KBDBLink	*dbLink
+	(	KBDBLink	*dbLink,
+		bool		pretty
 	)
 	const
 {
@@ -361,7 +363,14 @@ QString	KBSelectExpr::exprText
 		expr	= dbLink->mapExpression (expr) ;
 
 	if (!m_alias.isEmpty())
-		return	QString("%1 as %2").arg(expr).arg(m_alias) ;
+		expr	= QString("%1 as %2").arg(expr).arg(m_alias) ;
+
+	if (pretty)
+	{
+		expr.replace ("&", "&amp;") ;
+		expr.replace ("<", "&lt;") ;
+		expr.replace (">", "&gt;") ;
+	}
 
 	return	expr	;
 }
@@ -538,7 +547,7 @@ QString	KBSelect::getQueryText
 
 	for (uint idx2 = 0 ; idx2 < m_exprList.count() ; idx2 += 1)
 	{	SQL	+= sep	;
-		SQL	+= m_exprList[idx2].exprText(dbLink) ;
+		SQL	+= m_exprList[idx2].exprText(dbLink, pretty) ;
 		sep	 = ", "	;
 	}
 
@@ -596,15 +605,15 @@ QString	KBSelect::getQueryText
 
 	for (uint idx7 = 0 ; idx7 < m_whereList.count() ; idx7 += 1)
 	{	SQL	+= sep ;
-		SQL	+= m_whereList[idx7].exprText(dbLink) ;
-		sep	 = wrapBold	 (" and ", pretty) ;
+		SQL	+= m_whereList[idx7].exprText(dbLink, pretty) ;
+		sep	 = wrapBold (" and ", pretty) ;
 		if (pretty) SQL += "<br/>" ;
 	}
 
 	sep	= wrapBold(" group by ", pretty) ;
 	for (uint idx8 = 0 ; idx8 < m_groupList.count() ; idx8 += 1)
 	{	SQL	+= sep ;
-		SQL	+= m_groupList[idx8].exprText(dbLink) ;
+		SQL	+= m_groupList[idx8].exprText(dbLink, pretty) ;
 		sep	 = ", " ;
 	}
 	if (pretty && (m_groupList.count() > 0)) SQL += "<br/>" ;
@@ -612,15 +621,15 @@ QString	KBSelect::getQueryText
 	sep	= wrapBold(" having ", pretty) ;
 	for (uint idx9 = 0 ; idx9 < m_havingList.count() ; idx9 += 1)
 	{	SQL	+= sep ;
-		SQL	+= m_havingList[idx9].exprText(dbLink) ;
-		sep	 = ", " ;
+		SQL	+= m_havingList[idx9].exprText(dbLink, pretty) ;
+		sep	 = wrapBold (" and ", pretty) ;
 	}
 	if (pretty && (m_havingList.count() > 0)) SQL += "<br/>" ;
 
 	sep	= wrapBold(" order by ", pretty) ;
 	for (uint idx10 = 0 ; idx10 < m_orderList.count() ; idx10 += 1)
 	{	SQL	+= sep ;
-		SQL	+= m_orderList[idx10].exprText(dbLink) ;
+		SQL	+= m_orderList[idx10].exprText(dbLink, pretty) ;
 		sep	 = ", " ;
 	}
 	if (pretty && (m_orderList.count() > 0)) SQL += "<br/>" ;
@@ -1115,7 +1124,7 @@ bool	KBSelect::parseQuery
 	if (m_token.lower() == "having")
 	{
 		nextToken () ;
-		parseExprList (m_havingList, ",", false) ;
+		parseExprList (m_havingList, "and", false) ;
 	}
 
 	if (m_token.lower() == "order")
@@ -1327,26 +1336,26 @@ KBQryLevel
 
 	for (uint idx2 = 0 ; idx2 < m_whereList .count() ; idx2 += 1)
 	{
-		if (idx2 > 0) where += " and " ;
+		if (idx2 > 0) where  += " and " ;
 		where	+= m_whereList [idx2].exprText(&dbLink) ;
 	}
 
 	for (uint idx3 = 0 ; idx3 < m_orderList .count() ; idx3 += 1)
 	{
-		if (idx3 > 0) where += ", " ;
-		m_orderList [idx3].exprText(&dbLink) ;
+		if (idx3 > 0) order  += ", " ;
+		order	+= m_orderList [idx3].exprText(&dbLink) ;
 	}
 
 	for (uint idx4 = 0 ; idx4 < m_groupList .count() ; idx4 += 1)
 	{
-		if (idx4 > 0) where += ", " ;
-		m_groupList [idx4].exprText(&dbLink) ;
+		if (idx4 > 0) group  += ", " ;
+		group	+= m_groupList [idx4].exprText(&dbLink) ;
 	}
 
 	for (uint idx5 = 0 ; idx5 < m_havingList.count() ; idx5 += 1)
 	{
-		if (idx5 > 0) where += ", " ;
-		m_havingList[idx5].exprText(&dbLink) ;
+		if (idx5 > 0) having += " and " ;
+		having	+= m_havingList[idx5].exprText(&dbLink) ;
 	}
 
 	qryLevel->setGlobalWhere  (where ) ;
