@@ -822,13 +822,15 @@ void	KBTabber::setCurrentPage
 /*  offset	: QPoint	: Offset into writer			*/
 /*  first	: bool		: First control flag			*/
 /*  extra	: int &		: Return extra space			*/
+/*  prior	: bool		: Write prior value			*/
 /*  (returns)	: bool		: Success				*/
 
 bool	KBTabber::write
 	(	KBWriter	*writer,
 		QPoint		offset,
 		bool		first,
-		int		&extra
+		int		&extra,
+		bool		prior
 	)
 {
 	QString	bgcolor	;
@@ -847,7 +849,7 @@ bool	KBTabber::write
 	if (m_tabberBar->getCurrentTab (page) >= 0)
 		page->write  (writer, offset, first, extra) ;
 
-	m_tabberBar->write     (writer, offset, first, extra) ;
+	m_tabberBar->write     (writer, offset, first, extra, prior) ;
 	writer     ->setOffset (true, s)  ;
 
 	return	true	;
