@@ -163,6 +163,8 @@ bool	KBForm::moveFocusOK
 	if (!cBlk->focusOutOK ((qrow != m_formCurQRow) || (nBlk != cBlk)))
 		return	false  ;
 
+	m_formCurItem = 0 ;
+	m_formCurQRow = 0 ;
 	setUnMorphedItem(0, 0) ;
 	if (nBlk != cBlk) nBlk->enterBlock(false, 0) ;
 
