@@ -126,7 +126,7 @@ KBReportBlock::KBReportBlock
 	ySort.setAutoDelete (true) ;
 }
 
-/*  KBReportBlock								*/
+/*  KBReportBlock							*/
 /*  ~KBReportBlock: Destructor for form block element			*/
 /*  (returns)	: void		:					*/
 
@@ -415,7 +415,7 @@ bool	KBReportBlock::showData ()
 		if (getParent() == 0)
 			writer->setOffset (true, QPoint(0, writer->reserve (0))) ;
 
-		footer->writeData () ;
+		footer->writeData (false) ;
 		writer->setOffset (false, QPoint(0,footer->height())) ;
 
 		/* In case we are not the topmost block, "unreserve"	*/
@@ -462,7 +462,7 @@ void	KBReportBlock::finishPage
 
 	if (footer != 0)
 	{
-		footer->writeData () ;
+		footer->writeData (true) ;
 		writer->setOffset (false, QPoint(0, footer->height())) ;
 	}
 
