@@ -793,7 +793,12 @@ bool	KBQryDesign::saveRow
 		uint	qrow
 	)
 {
-	fprintf	(stderr, "KBQryDesign::saveRow(%u,%u)\n", qlvl, qrow) ;
+	fprintf
+	(	stderr, 
+		"KBQryDesign::saveRow: %u,%u\n",
+		qlvl,
+		qrow
+	)	;
 
 	bool		changed	= false	;
 	KBFieldSpec	*fspec	= 0	;
@@ -875,6 +880,13 @@ bool	KBQryDesign::saveRow
 			fspec	= newSpec.m_fldList.at (curQRow) ;
 			tc	= designInfo       .at (curQRow) ;
 
+			fprintf
+			(	stderr,
+				"KBQryDesign::saveRow: l=1 fspec=%p tc=%p\n",
+				(void *)fspec,
+				(void *)tc
+			)	;
+	
 			LITER
 			(	KBItem,
 				tblInner,
@@ -914,6 +926,9 @@ bool	KBQryDesign::saveRow
 			if (prec   != fspec->m_prec  ) changed = true ;
 			fspec->m_prec   = prec   ;
 
+			if (tc == 0)
+				break	;
+
 			LITER
 			(	KBItem,
 				tblInner,
@@ -926,23 +941,26 @@ bool	KBQryDesign::saveRow
 				value	= field->getValue (0) ;
 				if (value.getRawText() != tc->designValue (colIdx)) changed = true ;
 
-				fprintf	(stderr, "Save [%s] <- [%s]\n",
-						 (cchar *)field->getAttrVal("name"),
-						 (cchar *)field->getValue(0).getRawText()) ;
+				fprintf
+				(	stderr,
+					"KBQryDesign::saveRow: save [%s] <- [%s]\n",
+					(cchar *)field->getAttrVal("name"),
+					(cchar *)field->getValue(0).getRawText()
+				)	;
 
 				tc->setDesignValue (colIdx, value.getRawText()) ;
 			)
 
-		break	;
+			break	;
 
-	default	:
-		/* Any other query level is a fatal error ....		*/
-		KBError::EFault
-		(	QString (TR("Unexpected query level %1")).arg(qlvl),
-			QString::null,
-			__ERRLOCN
-		)	;
-		break	;
+		default	:
+			/* Any other query level is a fatal error ....		*/
+			KBError::EFault
+			(	QString (TR("Unexpected query level %1")).arg(qlvl),
+				QString::null,
+				__ERRLOCN
+			)	;
+			break	;
 	}
 
 	/* The changed flag is set for any changes. If set then set	*/
@@ -1345,8 +1363,8 @@ bool	KBQryDesign::syncAll
 	/* the design information.					*/
 	if (sDelta)
 	{
-		tmpName	= QString("__T_%1_%2").arg(time(0) % 1000000).arg(tabName) ;
-		oldName	= QString("__O_%1_%2").arg(time(0) % 1000000).arg(tabName) ;
+		tmpName	= dbLink.rekallPrefix(QString("T_%1_%2").arg(time(0) % 1000000).arg(tabName)) ;
+		oldName	= dbLink.rekallPrefix(QString("O_%1_%2").arg(time(0) % 1000000).arg(tabName)) ;
 
 		dbLink.dropTable (tmpName, true) ;
 
@@ -1397,7 +1415,11 @@ bool	KBQryDesign::syncAll
 				error,
 				true
 			))
-			error.DISPLAY() ;
+		{
+			setError (error) ;
+			finish	 (true ) ;
+			return	 false	 ;
+		}
 	}
 
 
