@@ -6,7 +6,7 @@
 
 (	cd lib
 	mkdir	-p .tmp
-	../../../sip/sip/sip -c .tmp rekall.sip
+	../../../sip/sip/sip -c .tmp rekall.sip || exit
 	cd	.tmp
 
 	for f in *
